@@ -194,9 +194,43 @@ static gboolean player_buffering_timeout (gpointer data);
 /* Logic */
 
 #ifdef CLUTTER_GST_ENABLE_DEBUG
+static gchar *
+get_stream_description (GstTagList *tags,
+                        gint        track_num)
+{
+  gchar *description = NULL;
+
+  if (tags)
+    {
+
+      gst_tag_list_get_string (tags, GST_TAG_LANGUAGE_CODE, &description);
+
+      if (description)
+        {
+          const gchar *language = gst_tag_get_language_name (description);
+
+          if (language)
+            {
+              g_free (description);
+              description = g_strdup (language);
+            }
+        }
+
+      if (!description)
+        gst_tag_list_get_string (tags, GST_TAG_CODEC, &description);
+    }
+
+  if (!description)
+    description = g_strdup_printf ("Track %d", track_num);
+
+  return description;
+}
+
 gchar *
 list_to_string (GList *list)
 {
+  GstTagList *tags;
+  gchar *description;
   GString *string;
   GList *l;
   gint n, i;
@@ -207,9 +241,17 @@ list_to_string (GList *list)
   string = g_string_new (NULL);
   n = g_list_length (list);
   for (i = 0, l = list; i < n - 1; i++, l = g_list_next (l))
-    g_string_append_printf (string, "%s, ", (gchar *) l->data);
+    {
+      tags = l->data;
+      description = get_stream_description (tags, i);
+      g_string_append_printf (string, "%s, ", description);
+      g_free (description);
+    }
 
-  g_string_append_printf (string, "%s", (gchar *) l->data);
+  tags = l->data;
+  description = get_stream_description (tags, i);
+  g_string_append_printf (string, "%s", (gchar *) description);
+  g_free (description);
 
   return g_string_free (string, FALSE);
 }
@@ -236,14 +278,14 @@ gst_state_to_string (GstState state)
 }
 
 static void
-free_string_list (GList **listp)
+free_tags_list (GList **listp)
 {
   GList *l;
 
   l = *listp;
   while (l)
     {
-      g_free (l->data);
+      gst_tag_list_free (l->data);
       l = g_list_delete_link (l, l);
     }
 
@@ -443,7 +485,6 @@ set_uri (ClutterGstPlayer *player,
 
       /* try to load subtitles based on the uri of the file */
       set_subtitle_uri (player, NULL);
-      autoload_subtitle (player, uri);
 
       /* reset the states of download buffering */
       player_clear_download_buffering (player);
@@ -452,8 +493,6 @@ set_uri (ClutterGstPlayer *player,
     {
       priv->uri = NULL;
 
-      set_subtitle_uri (player, NULL);
-
       if (priv->tick_timeout_id)
 	{
 	  g_source_remove (priv->tick_timeout_id);
@@ -490,6 +529,8 @@ set_uri (ClutterGstPlayer *player,
       gst_element_set_state (priv->pipeline, GST_STATE_NULL);
 
       g_object_set (priv->pipeline, "uri", uri, NULL);
+      set_subtitle_uri (player, NULL);
+      autoload_subtitle (player, uri);
 
       gst_element_set_state (priv->pipeline, state);
 
@@ -498,6 +539,7 @@ set_uri (ClutterGstPlayer *player,
   else
     {
       priv->is_idle = TRUE;
+      set_subtitle_uri (player, NULL);
       gst_element_set_state (priv->pipeline, GST_STATE_NULL);
       g_object_notify (G_OBJECT (player), "idle");
     }
@@ -511,11 +553,11 @@ set_uri (ClutterGstPlayer *player,
   g_object_notify (self, "duration");
   g_object_notify (self, "progress");
 
-  free_string_list (&priv->audio_streams);
+  free_tags_list (&priv->audio_streams);
   CLUTTER_GST_NOTE (AUDIO_STREAM, "audio-streams changed");
   g_object_notify (self, "audio-streams");
 
-  free_string_list (&priv->subtitle_tracks);
+  free_tags_list (&priv->subtitle_tracks);
   CLUTTER_GST_NOTE (SUBTITLES, "subtitle-tracks changed");
   g_object_notify (self, "subtitle-tracks");
 }
@@ -854,12 +896,23 @@ bus_message_error_cb (GstBus           *bus,
   g_object_notify (G_OBJECT (player), "idle");
 }
 
+/*
+ * This is what's intented in the EOS callback:
+ *   - receive EOS from playbin 2
+ *   - fire the EOS signal, the user can install a signal handler to loop the
+ *     video for instance.
+ *   - after having emitted the signal, check the state of the pipeline
+ *   - if the pipeline has been set back to playing or pause, don't touch the
+ *     idle state. This will avoid drawing a frame (or more) with the idle
+ *     material when looping
+ */
 static void
 bus_message_eos_cb (GstBus           *bus,
                     GstMessage       *message,
                     ClutterGstPlayer *player)
 {
   ClutterGstPlayerPrivate *priv = PLAYER_GET_PRIVATE (player);
+  GstState state, pending;
 
   priv->in_eos = TRUE;
 
@@ -868,8 +921,15 @@ bus_message_eos_cb (GstBus           *bus,
   g_signal_emit_by_name (player, "eos");
   g_object_notify (G_OBJECT (player), "progress");
 
-  priv->is_idle = TRUE;
-  g_object_notify (G_OBJECT (player), "idle");
+  gst_element_get_state (priv->pipeline, &state, &pending, 0);
+  if (pending)
+    state = pending;
+
+  if (!(state == GST_STATE_PLAYING || state == GST_STATE_PAUSED))
+    {
+      priv->is_idle = TRUE;
+      g_object_notify (G_OBJECT (player), "idle");
+    }
 }
 
 static void
@@ -1005,7 +1065,7 @@ bus_message_duration_cb (GstBus           *bus,
   /* GstElements send a duration message on the bus with GST_CLOCK_TIME_NONE
    * as duration to signal a new duration */
   gst_message_parse_duration (message, NULL, &duration);
-  if (G_UNLIKELY (duration != GST_CLOCK_TIME_NONE))
+  if (G_UNLIKELY ((GstClockTime) duration != GST_CLOCK_TIME_NONE))
     return;
 
   query_duration (player);
@@ -1148,7 +1208,7 @@ get_tags (GstElement  *pipeline,
           const gchar *action_signal)
 {
   GList *ret = NULL;
-  gint num = 1, i, n;
+  gint i, n;
 
   g_object_get (G_OBJECT (pipeline), property_name, &n, NULL);
   if (n == 0)
@@ -1157,93 +1217,27 @@ get_tags (GstElement  *pipeline,
   for (i = 0; i < n; i++)
     {
       GstTagList *tags = NULL;
-      gchar *description = NULL;
 
       g_signal_emit_by_name (G_OBJECT (pipeline), action_signal, i, &tags);
 
-      if (tags)
-        {
-
-          gst_tag_list_get_string (tags, GST_TAG_LANGUAGE_CODE, &description);
-
-          if (description)
-            {
-              const gchar *language = gst_tag_get_language_name (description);
-
-              if (language)
-                {
-                  g_free (description);
-                  description = g_strdup (language);
-                }
-            }
-
-          if (!description)
-            gst_tag_list_get_string (tags, GST_TAG_CODEC, &description);
-
-          gst_tag_list_free (tags);
-        }
-
-      if (!description)
-        description = g_strdup_printf ("Track #%d", num++);
-
-      ret = g_list_prepend (ret, description);
-
+      ret = g_list_prepend (ret, tags);
     }
 
   return g_list_reverse (ret);
 }
 
-static gboolean
-are_lists_equal (GList *list1,
-                 GList *list2)
-{
-  GList *l1, *l2;
-
-  l1 = list1;
-  l2 = list2;
-
-  while (l1)
-    {
-      const gchar *str1, *str2;
-
-      if (l2 == NULL)
-        return FALSE;
-
-      str1 = l1->data;
-      str2 = l2->data;
-
-      if (g_strcmp0 (str1, str2) != 0)
-        return FALSE;
-
-      l1 = g_list_next (l1);
-      l2 = g_list_next (l2);
-    }
-
-  return l2 == NULL;
-}
-
 static gboolean
 on_audio_changed_main_context (gpointer data)
 {
   ClutterGstPlayer *player = CLUTTER_GST_PLAYER (data);
   ClutterGstPlayerPrivate *priv = PLAYER_GET_PRIVATE (player);
-  GList *audio_streams;
 
-  audio_streams = get_tags (priv->pipeline, "n-audio", "get-audio-tags");
+  free_tags_list (&priv->audio_streams);
+  priv->audio_streams = get_tags (priv->pipeline, "n-audio", "get-audio-tags");
 
-  if (!are_lists_equal (priv->audio_streams, audio_streams))
-    {
-      free_string_list (&priv->audio_streams);
-      priv->audio_streams = audio_streams;
+  CLUTTER_GST_NOTE (AUDIO_STREAM, "audio-streams changed");
 
-      CLUTTER_GST_NOTE (AUDIO_STREAM, "audio-streams changed");
-
-      g_object_notify (G_OBJECT (player), "audio-streams");
-    }
-  else
-    {
-      free_string_list (&audio_streams);
-    }
+  g_object_notify (G_OBJECT (player), "audio-streams");
 
   return FALSE;
 }
@@ -1261,6 +1255,13 @@ on_audio_tags_changed (GstElement       *pipeline,
                        gint              stream,
                        ClutterGstPlayer *player)
 {
+  gint current_stream;
+
+  g_object_get (G_OBJECT (pipeline), "current-audio", &current_stream, NULL);
+
+  if (current_stream != stream)
+    return;
+
   g_idle_add (on_audio_changed_main_context, player);
 }
 
@@ -1288,23 +1289,13 @@ on_text_changed_main_context (gpointer data)
 {
   ClutterGstPlayer *player = CLUTTER_GST_PLAYER (data);
   ClutterGstPlayerPrivate *priv = PLAYER_GET_PRIVATE (player);
-  GList *subtitle_tracks;
 
-  subtitle_tracks = get_tags (priv->pipeline, "n-text", "get-text-tags");
+  free_tags_list (&priv->subtitle_tracks);
+  priv->subtitle_tracks = get_tags (priv->pipeline, "n-text", "get-text-tags");
 
-  if (!are_lists_equal (priv->subtitle_tracks, subtitle_tracks))
-    {
-      free_string_list (&priv->subtitle_tracks);
-      priv->subtitle_tracks = subtitle_tracks;
+  CLUTTER_GST_NOTE (AUDIO_STREAM, "subtitle-tracks changed");
 
-      CLUTTER_GST_NOTE (AUDIO_STREAM, "subtitle-tracks changed");
-
-      g_object_notify (G_OBJECT (player), "subtitle-tracks");
-    }
-  else
-    {
-      free_string_list (&subtitle_tracks);
-    }
+  g_object_notify (G_OBJECT (player), "subtitle-tracks");
 
   return FALSE;
 }
@@ -1617,6 +1608,257 @@ get_pipeline (void)
   return pipeline;
 }
 
+/* ClutterGstPlayerIface implementation */
+
+static GstElement *
+clutter_gst_player_get_pipeline_impl (ClutterGstPlayer *player)
+{
+  ClutterGstPlayerPrivate *priv;
+
+  priv = PLAYER_GET_PRIVATE (player);
+
+  return priv->pipeline;
+}
+
+static gchar *
+clutter_gst_player_get_user_agent_impl (ClutterGstPlayer *player)
+{
+  ClutterGstPlayerPrivate *priv;
+  GstElement *source;
+  GParamSpec *pspec;
+  gchar *user_agent;
+
+  priv = PLAYER_GET_PRIVATE (player);
+
+  /* If the user has set a custom user agent, we just return it even if it is
+   * not used by the current source element of the pipeline */
+  if (priv->user_agent)
+    return g_strdup (priv->user_agent);
+
+  /* If not, we try to retrieve the user agent used by the current source */
+  g_object_get (priv->pipeline, "source", &source, NULL);
+  if (source == NULL)
+    return NULL;
+
+  pspec = g_object_class_find_property (G_OBJECT_GET_CLASS (source),
+                                        "user-agent");
+  if (pspec == NULL)
+    return NULL;
+
+  g_object_get (source, "user-agent", &user_agent, NULL);
+
+  return user_agent;
+}
+
+static void
+clutter_gst_player_set_user_agent_impl (ClutterGstPlayer *player,
+                                        const gchar      *user_agent)
+{
+  ClutterGstPlayerPrivate *priv;
+
+  priv = PLAYER_GET_PRIVATE (player);
+
+  g_free (priv->user_agent);
+  if (user_agent)
+    priv->user_agent = g_strdup (user_agent);
+  else
+    priv->user_agent = NULL;
+
+  player_set_user_agent (player, user_agent);
+}
+
+static ClutterGstSeekFlags
+clutter_gst_player_get_seek_flags_impl (ClutterGstPlayer *player)
+{
+  ClutterGstPlayerPrivate *priv;
+
+  priv = PLAYER_GET_PRIVATE (player);
+
+  if (priv->seek_flags == GST_SEEK_FLAG_ACCURATE)
+    return CLUTTER_GST_SEEK_FLAG_ACCURATE;
+  else
+    return CLUTTER_GST_SEEK_FLAG_NONE;
+}
+
+static void
+clutter_gst_player_set_seek_flags_impl (ClutterGstPlayer    *player,
+                                        ClutterGstSeekFlags  flags)
+{
+  ClutterGstPlayerPrivate *priv;
+
+  priv = PLAYER_GET_PRIVATE (player);
+
+  if (flags == CLUTTER_GST_SEEK_FLAG_NONE)
+    priv->seek_flags = GST_SEEK_FLAG_KEY_UNIT;
+  else if (flags & CLUTTER_GST_SEEK_FLAG_ACCURATE)
+    priv->seek_flags = GST_SEEK_FLAG_ACCURATE;
+}
+
+static ClutterGstBufferingMode
+clutter_gst_player_get_buffering_mode_impl (ClutterGstPlayer *player)
+{
+  ClutterGstPlayerPrivate *priv;
+  GstPlayFlags flags;
+
+  priv = PLAYER_GET_PRIVATE (player);
+
+  g_object_get (G_OBJECT (priv->pipeline), "flags", &flags, NULL);
+
+  if (flags & GST_PLAY_FLAG_DOWNLOAD)
+    return CLUTTER_GST_BUFFERING_MODE_DOWNLOAD;
+
+  return CLUTTER_GST_BUFFERING_MODE_STREAM;
+}
+
+static void
+clutter_gst_player_set_buffering_mode_impl (ClutterGstPlayer        *player,
+                                            ClutterGstBufferingMode  mode)
+{
+  ClutterGstPlayerPrivate *priv;
+  GstPlayFlags flags;
+
+  priv = PLAYER_GET_PRIVATE (player);
+
+  g_object_get (G_OBJECT (priv->pipeline), "flags", &flags, NULL);
+
+  switch (mode)
+    {
+    case CLUTTER_GST_BUFFERING_MODE_STREAM:
+      flags &= ~GST_PLAY_FLAG_DOWNLOAD;
+      break;
+
+    case CLUTTER_GST_BUFFERING_MODE_DOWNLOAD:
+      flags |= GST_PLAY_FLAG_DOWNLOAD;
+      break;
+
+    default:
+      g_warning ("Unexpected buffering mode %d", mode);
+      break;
+    }
+
+  g_object_set (G_OBJECT (priv->pipeline), "flags", flags, NULL);
+}
+
+static GList *
+clutter_gst_player_get_audio_streams_impl (ClutterGstPlayer *player)
+{
+  ClutterGstPlayerPrivate *priv;
+
+  priv = PLAYER_GET_PRIVATE (player);
+
+  if (CLUTTER_GST_DEBUG_ENABLED (AUDIO_STREAM))
+    {
+      gchar *streams;
+
+      streams = list_to_string (priv->audio_streams);
+      CLUTTER_GST_NOTE (AUDIO_STREAM, "audio streams: %s", streams);
+      g_free (streams);
+    }
+
+  return priv->audio_streams;
+}
+
+static gint
+clutter_gst_player_get_audio_stream_impl (ClutterGstPlayer *player)
+{
+  ClutterGstPlayerPrivate *priv;
+  gint index_ = -1;
+
+  priv = PLAYER_GET_PRIVATE (player);
+
+  g_object_get (G_OBJECT (priv->pipeline),
+                "current-audio", &index_,
+                NULL);
+
+  CLUTTER_GST_NOTE (AUDIO_STREAM, "audio stream is #%d", index_);
+
+  return index_;
+}
+
+static void
+clutter_gst_player_set_audio_stream_impl (ClutterGstPlayer *player,
+                                          gint              index_)
+{
+  ClutterGstPlayerPrivate *priv;
+
+  priv = PLAYER_GET_PRIVATE (player);
+
+  g_return_if_fail (index_ >= 0 &&
+                    index_ < (gint) g_list_length (priv->audio_streams));
+
+  CLUTTER_GST_NOTE (AUDIO_STREAM, "set audio audio stream to #%d", index_);
+
+  g_object_set (G_OBJECT (priv->pipeline),
+                "current-audio", index_,
+                NULL);
+}
+
+static GList *
+clutter_gst_player_get_subtitle_tracks_impl (ClutterGstPlayer *player)
+{
+  ClutterGstPlayerPrivate *priv;
+
+  priv = PLAYER_GET_PRIVATE (player);
+
+  if (CLUTTER_GST_DEBUG_ENABLED (SUBTITLES))
+    {
+      gchar *tracks;
+
+      tracks = list_to_string (priv->subtitle_tracks);
+      CLUTTER_GST_NOTE (SUBTITLES, "subtitle tracks: %s", tracks);
+      g_free (tracks);
+    }
+
+  return priv->subtitle_tracks;
+}
+
+static gint
+clutter_gst_player_get_subtitle_track_impl (ClutterGstPlayer *player)
+{
+  ClutterGstPlayerPrivate *priv;
+  gint index_ = -1;
+
+  priv = PLAYER_GET_PRIVATE (player);
+
+  g_object_get (G_OBJECT (priv->pipeline),
+                "current-text", &index_,
+                NULL);
+
+  CLUTTER_GST_NOTE (SUBTITLES, "text track is #%d", index_);
+
+  return index_;
+}
+
+static void
+clutter_gst_player_set_subtitle_track_impl (ClutterGstPlayer *player,
+                                            gint              index_)
+{
+  ClutterGstPlayerPrivate *priv;
+
+  priv = PLAYER_GET_PRIVATE (player);
+
+  g_return_if_fail (index_ >= -1 &&
+                    index_ < (gint) g_list_length (priv->subtitle_tracks));
+
+  CLUTTER_GST_NOTE (SUBTITLES, "set subtitle track to #%d", index_);
+
+  g_object_set (G_OBJECT (priv->pipeline),
+                "current-text", index_,
+                NULL);
+}
+
+static gboolean
+clutter_gst_player_get_idle_impl (ClutterGstPlayer *player)
+{
+  ClutterGstPlayerPrivate *priv;
+
+  priv = PLAYER_GET_PRIVATE (player);
+
+  return priv->is_idle;
+}
+
+/**/
+
 /**
  * clutter_gst_player_init:
  * @player: a #ClutterGstPlayer
@@ -1637,6 +1879,7 @@ gboolean
 clutter_gst_player_init (ClutterGstPlayer *player)
 {
   ClutterGstPlayerPrivate *priv;
+  ClutterGstPlayerIface *iface;
 
   g_return_val_if_fail (CLUTTER_GST_IS_PLAYER (player), FALSE);
 
@@ -1644,6 +1887,23 @@ clutter_gst_player_init (ClutterGstPlayer *player)
   if (priv)
     return TRUE;
 
+  iface = CLUTTER_GST_PLAYER_GET_INTERFACE (player);
+
+  iface->get_pipeline = clutter_gst_player_get_pipeline_impl;
+  iface->get_user_agent = clutter_gst_player_get_user_agent_impl;
+  iface->set_user_agent = clutter_gst_player_set_user_agent_impl;
+  iface->get_seek_flags = clutter_gst_player_get_seek_flags_impl;
+  iface->set_seek_flags = clutter_gst_player_set_seek_flags_impl;
+  iface->get_buffering_mode = clutter_gst_player_get_buffering_mode_impl;
+  iface->set_buffering_mode = clutter_gst_player_set_buffering_mode_impl;
+  iface->get_audio_streams = clutter_gst_player_get_audio_streams_impl;
+  iface->get_audio_stream = clutter_gst_player_get_audio_stream_impl;
+  iface->set_audio_stream = clutter_gst_player_set_audio_stream_impl;
+  iface->get_subtitle_tracks = clutter_gst_player_get_subtitle_tracks_impl;
+  iface->get_subtitle_track = clutter_gst_player_get_subtitle_track_impl;
+  iface->set_subtitle_track = clutter_gst_player_set_subtitle_track_impl;
+  iface->get_idle = clutter_gst_player_get_idle_impl;
+
   priv = g_slice_new0 (ClutterGstPlayerPrivate);
   PLAYER_SET_PRIVATE (player, priv);
 
@@ -1756,8 +2016,8 @@ clutter_gst_player_deinit (ClutterGstPlayer *player)
 
   g_free (priv->uri);
   g_free (priv->font_name);
-  free_string_list (&priv->audio_streams);
-  free_string_list (&priv->subtitle_tracks);
+  free_tags_list (&priv->audio_streams);
+  free_tags_list (&priv->subtitle_tracks);
 
   g_slice_free (ClutterGstPlayerPrivate, priv);
 }
@@ -1883,6 +2143,8 @@ clutter_gst_player_default_init (ClutterGstPlayerIface *iface)
     }
 }
 
+/* ClutterGstIface */
+
 /**
  * clutter_gst_player_get_pipeline:
  * @player: a #ClutterGstPlayer
@@ -1897,13 +2159,13 @@ clutter_gst_player_default_init (ClutterGstPlayerIface *iface)
 GstElement *
 clutter_gst_player_get_pipeline (ClutterGstPlayer *player)
 {
-  ClutterGstPlayerPrivate *priv;
+  ClutterGstPlayerIface *iface;
 
   g_return_val_if_fail (CLUTTER_GST_IS_PLAYER (player), NULL);
 
-  priv = PLAYER_GET_PRIVATE (player);
+  iface = CLUTTER_GST_PLAYER_GET_INTERFACE (player);
 
-  return priv->pipeline;
+  return iface->get_pipeline (player);
 }
 
 /**
@@ -1920,33 +2182,13 @@ clutter_gst_player_get_pipeline (ClutterGstPlayer *player)
 gchar *
 clutter_gst_player_get_user_agent (ClutterGstPlayer *player)
 {
-  ClutterGstPlayerPrivate *priv;
-  GstElement *source;
-  GParamSpec *pspec;
-  gchar *user_agent;
+  ClutterGstPlayerIface *iface;
 
   g_return_val_if_fail (CLUTTER_GST_IS_PLAYER (player), NULL);
 
-  priv = PLAYER_GET_PRIVATE (player);
+  iface = CLUTTER_GST_PLAYER_GET_INTERFACE (player);
 
-  /* If the user has set a custom user agent, we just return it even if it is
-   * not used by the current source element of the pipeline */
-  if (priv->user_agent)
-    return g_strdup (priv->user_agent);
-
-  /* If not, we try to retrieve the user agent used by the current source */
-  g_object_get (priv->pipeline, "source", &source, NULL);
-  if (source == NULL)
-    return NULL;
-
-  pspec = g_object_class_find_property (G_OBJECT_GET_CLASS (source),
-                                        "user-agent");
-  if (pspec == NULL)
-    return NULL;
-
-  g_object_get (source, "user-agent", &user_agent, NULL);
-
-  return user_agent;
+  return iface->get_user_agent (player);
 }
 
 /**
@@ -1966,19 +2208,13 @@ void
 clutter_gst_player_set_user_agent (ClutterGstPlayer *player,
                                    const gchar      *user_agent)
 {
-  ClutterGstPlayerPrivate *priv;
+  ClutterGstPlayerIface *iface;
 
   g_return_if_fail (CLUTTER_GST_IS_PLAYER (player));
 
-  priv = PLAYER_GET_PRIVATE (player);
+  iface = CLUTTER_GST_PLAYER_GET_INTERFACE (player);
 
-  g_free (priv->user_agent);
-  if (user_agent)
-    priv->user_agent = g_strdup (user_agent);
-  else
-    priv->user_agent = NULL;
-
-  player_set_user_agent (player, user_agent);
+  iface->set_user_agent (player, user_agent);
 }
 
 /**
@@ -1994,17 +2230,14 @@ clutter_gst_player_set_user_agent (ClutterGstPlayer *player,
 ClutterGstSeekFlags
 clutter_gst_player_get_seek_flags (ClutterGstPlayer *player)
 {
-  ClutterGstPlayerPrivate *priv;
+  ClutterGstPlayerIface *iface;
 
   g_return_val_if_fail (CLUTTER_GST_IS_PLAYER (player),
                         CLUTTER_GST_SEEK_FLAG_NONE);
 
-  priv = PLAYER_GET_PRIVATE (player);
+  iface = CLUTTER_GST_PLAYER_GET_INTERFACE (player);
 
-  if (priv->seek_flags == GST_SEEK_FLAG_ACCURATE)
-    return CLUTTER_GST_SEEK_FLAG_ACCURATE;
-  else
-    return CLUTTER_GST_SEEK_FLAG_NONE;
+  return iface->get_seek_flags (player);
 }
 
 /**
@@ -2021,16 +2254,13 @@ void
 clutter_gst_player_set_seek_flags (ClutterGstPlayer    *player,
                                    ClutterGstSeekFlags  flags)
 {
-  ClutterGstPlayerPrivate *priv;
+  ClutterGstPlayerIface *iface;
 
   g_return_if_fail (CLUTTER_GST_IS_PLAYER (player));
 
-  priv = PLAYER_GET_PRIVATE (player);
+  iface = CLUTTER_GST_PLAYER_GET_INTERFACE (player);
 
-  if (flags == CLUTTER_GST_SEEK_FLAG_NONE)
-    priv->seek_flags = GST_SEEK_FLAG_KEY_UNIT;
-  else if (flags & CLUTTER_GST_SEEK_FLAG_ACCURATE)
-    priv->seek_flags = GST_SEEK_FLAG_ACCURATE;
+  iface->set_seek_flags (player, flags);
 }
 
 /**
@@ -2044,20 +2274,14 @@ clutter_gst_player_set_seek_flags (ClutterGstPlayer    *player,
 ClutterGstBufferingMode
 clutter_gst_player_get_buffering_mode (ClutterGstPlayer *player)
 {
-  ClutterGstPlayerPrivate *priv;
-  GstPlayFlags flags;
+  ClutterGstPlayerIface *iface;
 
   g_return_val_if_fail (CLUTTER_GST_IS_PLAYER (player),
                         CLUTTER_GST_BUFFERING_MODE_STREAM);
 
-  priv = PLAYER_GET_PRIVATE (player);
+  iface = CLUTTER_GST_PLAYER_GET_INTERFACE (player);
 
-  g_object_get (G_OBJECT (priv->pipeline), "flags", &flags, NULL);
-
-  if (flags & GST_PLAY_FLAG_DOWNLOAD)
-    return CLUTTER_GST_BUFFERING_MODE_DOWNLOAD;
-
-  return CLUTTER_GST_BUFFERING_MODE_STREAM;
+  return iface->get_buffering_mode (player);
 }
 
 /**
@@ -2071,31 +2295,13 @@ void
 clutter_gst_player_set_buffering_mode (ClutterGstPlayer        *player,
                                        ClutterGstBufferingMode  mode)
 {
-  ClutterGstPlayerPrivate *priv;
-  GstPlayFlags flags;
+  ClutterGstPlayerIface *iface;
 
   g_return_if_fail (CLUTTER_GST_IS_PLAYER (player));
 
-  priv = PLAYER_GET_PRIVATE (player);
+  iface = CLUTTER_GST_PLAYER_GET_INTERFACE (player);
 
-  g_object_get (G_OBJECT (priv->pipeline), "flags", &flags, NULL);
-
-  switch (mode)
-    {
-    case CLUTTER_GST_BUFFERING_MODE_STREAM:
-      flags &= ~GST_PLAY_FLAG_DOWNLOAD;
-      break;
-
-    case CLUTTER_GST_BUFFERING_MODE_DOWNLOAD:
-      flags |= GST_PLAY_FLAG_DOWNLOAD;
-      break;
-
-    default:
-      g_warning ("Unexpected buffering mode %d", mode);
-      break;
-    }
-
-  g_object_set (G_OBJECT (priv->pipeline), "flags", flags, NULL);
+  iface->set_buffering_mode (player, mode);
 }
 
 /**
@@ -2112,22 +2318,13 @@ clutter_gst_player_set_buffering_mode (ClutterGstPlayer        *player,
 GList *
 clutter_gst_player_get_audio_streams (ClutterGstPlayer *player)
 {
-  ClutterGstPlayerPrivate *priv;
+  ClutterGstPlayerIface *iface;
 
   g_return_val_if_fail (CLUTTER_GST_IS_PLAYER (player), NULL);
 
-  priv = PLAYER_GET_PRIVATE (player);
+  iface = CLUTTER_GST_PLAYER_GET_INTERFACE (player);
 
-  if (CLUTTER_GST_DEBUG_ENABLED (AUDIO_STREAM))
-    {
-      gchar *streams;
-
-      streams = list_to_string (priv->audio_streams);
-      CLUTTER_GST_NOTE (AUDIO_STREAM, "audio streams: %s", streams);
-      g_free (streams);
-    }
-
-  return priv->audio_streams;
+  return iface->get_audio_streams (player);
 }
 
 /**
@@ -2146,20 +2343,13 @@ clutter_gst_player_get_audio_streams (ClutterGstPlayer *player)
 gint
 clutter_gst_player_get_audio_stream (ClutterGstPlayer *player)
 {
-  ClutterGstPlayerPrivate *priv;
-  gint index_ = -1;
+  ClutterGstPlayerIface *iface;
 
   g_return_val_if_fail (CLUTTER_GST_IS_PLAYER (player), -1);
 
-  priv = PLAYER_GET_PRIVATE (player);
+  iface = CLUTTER_GST_PLAYER_GET_INTERFACE (player);
 
-  g_object_get (G_OBJECT (priv->pipeline),
-                "current-audio", &index_,
-                NULL);
-
-  CLUTTER_GST_NOTE (AUDIO_STREAM, "audio stream is #%d", index_);
-
-  return index_;
+  return iface->get_audio_stream (player);
 }
 
 /**
@@ -2176,20 +2366,13 @@ void
 clutter_gst_player_set_audio_stream (ClutterGstPlayer *player,
                                      gint              index_)
 {
-  ClutterGstPlayerPrivate *priv;
+  ClutterGstPlayerIface *iface;
 
   g_return_if_fail (CLUTTER_GST_IS_PLAYER (player));
 
-  priv = PLAYER_GET_PRIVATE (player);
+  iface = CLUTTER_GST_PLAYER_GET_INTERFACE (player);
 
-  g_return_if_fail (index_ >= 0 &&
-                    index_ < g_list_length (priv->audio_streams));
-
-  CLUTTER_GST_NOTE (AUDIO_STREAM, "set audio audio stream to #%d", index_);
-
-  g_object_set (G_OBJECT (priv->pipeline),
-                "current-audio", index_,
-                NULL);
+  iface->set_audio_stream (player, index_);
 }
 
 /**
@@ -2206,22 +2389,13 @@ clutter_gst_player_set_audio_stream (ClutterGstPlayer *player,
 GList *
 clutter_gst_player_get_subtitle_tracks (ClutterGstPlayer *player)
 {
-  ClutterGstPlayerPrivate *priv;
+  ClutterGstPlayerIface *iface;
 
   g_return_val_if_fail (CLUTTER_GST_IS_PLAYER (player), NULL);
 
-  priv = PLAYER_GET_PRIVATE (player);
+  iface = CLUTTER_GST_PLAYER_GET_INTERFACE (player);
 
-  if (CLUTTER_GST_DEBUG_ENABLED (SUBTITLES))
-    {
-      gchar *tracks;
-
-      tracks = list_to_string (priv->subtitle_tracks);
-      CLUTTER_GST_NOTE (SUBTITLES, "subtitle tracks: %s", tracks);
-      g_free (tracks);
-    }
-
-  return priv->subtitle_tracks;
+  return iface->get_subtitle_tracks (player);
 }
 
 /**
@@ -2229,7 +2403,7 @@ clutter_gst_player_get_subtitle_tracks (ClutterGstPlayer *player)
  * @player: a #ClutterGstPlayer
  *
  * Get the current subtitles track. The number returned is the index of the
- * subitles track in the list returned by
+ * subtiles track in the list returned by
  * clutter_gst_player_get_subtitle_tracks().
  *
  * Return value: the index of the current subtitlest track, -1 if the media has
@@ -2240,21 +2414,13 @@ clutter_gst_player_get_subtitle_tracks (ClutterGstPlayer *player)
 gint
 clutter_gst_player_get_subtitle_track (ClutterGstPlayer *player)
 {
-  ClutterGstPlayerPrivate *priv;
-  gint index_ = -1;
+  ClutterGstPlayerIface *iface;
 
   g_return_val_if_fail (CLUTTER_GST_IS_PLAYER (player), -1);
 
-  priv = PLAYER_GET_PRIVATE (player);
-
-  g_object_get (G_OBJECT (priv->pipeline),
-                "current-text", &index_,
-                NULL);
-
-  CLUTTER_GST_NOTE (SUBTITLES, "text track is #%d", index_);
-
-  return index_;
+  iface = CLUTTER_GST_PLAYER_GET_INTERFACE (player);
 
+  return iface->get_subtitle_track (player);
 }
 
 /**
@@ -2273,20 +2439,13 @@ void
 clutter_gst_player_set_subtitle_track (ClutterGstPlayer *player,
                                        gint              index_)
 {
-  ClutterGstPlayerPrivate *priv;
+  ClutterGstPlayerIface *iface;
 
   g_return_if_fail (CLUTTER_GST_IS_PLAYER (player));
 
-  priv = PLAYER_GET_PRIVATE (player);
+  iface = CLUTTER_GST_PLAYER_GET_INTERFACE (player);
 
-  g_return_if_fail (index_ >= -1 &&
-                    index_ < g_list_length (priv->subtitle_tracks));
-
-  CLUTTER_GST_NOTE (SUBTITLES, "set subtitle track to #%d", index_);
-
-  g_object_set (G_OBJECT (priv->pipeline),
-                "current-text", index_,
-                NULL);
+  iface->set_subtitle_track (player, index_);
 }
 
 /**
@@ -2302,11 +2461,11 @@ clutter_gst_player_set_subtitle_track (ClutterGstPlayer *player,
 gboolean
 clutter_gst_player_get_idle (ClutterGstPlayer *player)
 {
-  ClutterGstPlayerPrivate *priv;
+  ClutterGstPlayerIface *iface;
 
   g_return_val_if_fail (CLUTTER_GST_IS_PLAYER (player), TRUE);
 
-  priv = PLAYER_GET_PRIVATE (player);
+  iface = CLUTTER_GST_PLAYER_GET_INTERFACE (player);
 
-  return priv->is_idle;
+  return iface->get_idle (player);
 }
